% lab20_ex_rcos2.m
% Raised cosine (RC) and square root RC (SRRC) pulse shaping filters (PSF)
clear all; close all;

psf = 'rcos';               % 'rcos', 'sqrt-rcos'
Nsymb = 10;                 % number of samples per symbol, interpolation order
Ksymb = 8;                  % number of symbols per PSF filter 
Npsf = Ksymb*Nsymb;         % filter length
n = 0 : Npsf; Nc = Npsf/2;  % indexes of filter samples, index of the central sample

figure; ax1=[ -Nc, Nc, -0.25/Nsymb, 1.20/Nsymb ]; ax2=[ 0, 0.5, -120, 20 ];
txt = [];

for r = 0.1 : 0.2 : 0.9                 % filter roll-off factor
  if( isequal( psf, 'rcos') )           % definition of raised cosine
      hpsf = sin(pi*(n-Nc)/Nsymb) ./ (pi*(n-Nc)/Nsymb) .* ...
             cos(pi*r*(n-Nc)/Nsymb) ./ (1-(2*r*(n-Nc)/Nsymb).^2);
      hpsf(Nc+1) = 1;   
      n0 = find( abs(1-(2*r*(n-Nc)/Nsymb).^2 ) < sqrt(eps) );
      hpsf(n0) = r/2 * sin(pi/(2*r));
  elseif( isequal( psf, 'sqrt-rcos') )  % definition of sqrt of raised cosine
     n0 = find( abs(1-(4*r*(n-Nc)/Nsymb).^2) < 5*eps ), pause
     if( prod(size(n0)) > 0 ) disp('Div by 0. Change parameters'); end
     hpsf = 4*r/pi * ( cos(pi*(1+r)*(n-Nc)/Nsymb)+(Nsymb./(4*r*(n-Nc)) ) ...
            .* sin(pi*(1-r)*(n-Nc)/Nsymb) ) ./ (1-(4*r*(n-Nc)/Nsymb).^2);
     hpsf(Nc+1) = (1+r*(4/pi-1));   
  end
  hpsf = hpsf/Nsymb; % normalization
  
  subplot(211); plot( n-Nc, hpsf ); title('psf(n)'); axis( ax1 ); grid; hold on; 
  Nf=2000; fn=(0:Nf)/(2*Nf); om=2*pi*fn; z=exp(-j*om); Hpsf=polyval(hpsf,z);
  subplot(212); plot(fn,20*log10(abs(Hpsf))); xlabel('fn (Hz/Hz)'); ylabel('(dB)');
  title('|Hpsf(fn)| (dB)'); axis( ax2 ); grid; hold on
  txt = [ txt; strcat('r=',num2str(r)) ]; legend( txt ); pause
end

if(1) % Comparison with Matlab
   if( isequal( psf, 'rcos') ) type = 'normal'; else type = 'normal'; end    
   fs = 1; fcut = 1/(2*Nsymb)
   hpsf0 = firrcos( Npsf, fcut, r, fs,'rolloff',type); % MATLAB function
   error = max(abs(hpsf-hpsf0)), pause
   
   figure; plot(n-Nc,hpsf0,'ro',n-Nc,hpsf,'bx'); xlabel('n'); title('psf(n)');
   grid; pause
   Nf=1000; fn=(0:Nf)/(2*Nf); om=2*pi*fn; z=exp(-j*om); Hpsf=polyval(hpsf,z);
   figure; plot(fn,20*log10(abs(Hpsf))); xlabel('fn (Hz/Hz)'); ylabel('(dB)');
   title('| Hpsf(fn) | (dB)'); grid; pause
end
